-- ------------------------------------
-- MAP Exporter
-- ------------------------------------
-- exports MAP-Files
-- ------------------------------------

-----------------------------
-- Tools
-----------------------------

fn getLocalFaceRotation mesh index = ((matrixFromNormal (getFaceNormal mesh index)*mesh.transform ) as eulerAngles) 

fn existFile fname = (getfiles fname).count != 0

fn existDir fname = 
(
	temp=fname
	if temp[temp.count]=="\\" do temp=substring temp 1 (temp.count-1)
	if (getDirectories temp).count!=0 then result=true else result=false 
	return result
)

fn cutNumber num = if abs(num-int(num))<0.001 then return int(num) else return num

fn getFileAndLastDir filename =
(
	outName=(filterstring (getFilenamePath filename) "\\")[(filterstring (getFilenamePath filename) "\\").count]
	outName+="/"+(getFilenameFile filename) 
	return outName
)

fn cleanMesh mesh =
(
	addmodifier mesh (cap_holes())
	convertToMesh mesh
	meshop.removeDegenerateFaces mesh
	meshop.removeIllegalFaces mesh
	return mesh
)	

-----------------------------
-- Global definitions
-----------------------------

global vmID=1

fn createVersionInfo =
(
	versioninfo="versioninfo\n{\n"
	versioninfo+="  \"editorversion\" \"400\"\n"
	versioninfo+="  \"editorbuild\" \"2945\"\n"
	versioninfo+="  \"mapversion\" \"1\"\n"
	versioninfo+="  \"formatversion\" \"100\"\n"
	versioninfo+="  \"prefab\" \"0\"\n"
	versioninfo+="}\n"
)

-----------------------------
-- Geometry - SubObjects
-----------------------------

fn createUV obj face size =
(
	
	faceNormal=getFaceNormal obj face
	faceMatrix=matrixfromNormal faceNormal
	uCoords=faceMatrix[1]
	vCoords=faceMatrix[2]
	
	-- so far i create the UV settings from the face's normal.
	-- this doesn't take the actual UV set in account.
	-- Highly WIP!
	uv="  \"uaxis\" "
	uv+="\"["+(uCoords[1] as string)+" "+(uCoords[2] as string)+" "+(uCoords[3] as string)+" 0] 0.25\"\n"
	uv+="  \"vaxis\" "
	uv+="\"["+(vCoords[1] as string)+" "+(vCoords[2] as string)+" "+(vCoords[3] as string)+" 0] 0.25\"\n"
	
  return uv
)

fn createSide obj face size =
(
	-- obj must be mesh!
	
	side="side\n{\n"
	side+="  \"id\" \""+(vmID as string)+"\"\n"
	vmID+=1	-- needs to be increased after every instance

	-- plane definition: Vertices in counter-3dsmax-order
	fv=getFace obj face
	p1=getVert obj fv[1]
	p2=getVert obj fv[2]
	p3=getVert obj fv[3]
	
	side+="  \"plane\" \"("
	side+=(int(p3[1]*size) as string)+" "+(int(p3[2]*size) as string)+" "+(int(p3[3]*size) as string)+") ("
	side+=(int(p2[1]*size) as string)+" "+(int(p2[2]*size) as string)+" "+(int(p2[3]*size) as string)+") ("
	side+=(int(p1[1]*size) as string)+" "+(int(p1[2]*size) as string)+" "+(int(p1[3]*size) as string)+")\"\n"
	
	-- material definition
	case (classof obj.material) of
	(
		Standardmaterial:
		(
			if (classof obj.material.diffusemap)==Bitmaptexture then
			(
				tFilename=obj.material.diffusemap.filename
				tName=(getFileAndLastDir obj.material.diffusemap.filename)
			)
			else tName="DEV/DEV_MEASUREGENERIC01"
		)
		Multimaterial:
		(
			faceID=getFaceMatID obj face
			if (classof obj.material.materiallist[faceID].diffusemap)==Bitmaptexture then
			(
				tFilename=obj.material.materiallist[faceID].diffusemap.filename
				tName=(getFileAndLastDir obj.material.materiallist[faceID].diffusemap.filename)
			)
			else tName="DEV/DEV_MEASUREGENERIC01"
		)
		default: tName="DEV/DEV_MEASUREGENERIC01"
		
	)
	side+="  \"material\" \""+tName+"\"\n"

	-- uv
	side+=createUV obj face size
	
  side+="  \"rotation\" \"0\"\n"
  side+="  \"lightmapscale\" \"16\"\n"
  
  -- smoothing gropus
  side+="  \"smoothing_groups\" \""+((getFaceSmoothGroup obj face) as string)+"\"\n"
	
	side+="}\n"	
	return side
)

-----------------------------
-- Geometry
-----------------------------

-- Mesh
fn createSolidMesh obj size =
(
	solid=""
	face=#(1)
	while face.count<obj.numfaces do
	(
		oldFaceCount=0
		while face.count!=oldFaceCount do
		(
			oldFaceCount=face.count
			--face=(meshop.getVertsUsingFace obj face) as array
			face=(meshop.getFacesUsingVert obj (meshop.getVertsUsingFace obj face)) as array
		)
		
		-- start solid
		solid+="solid\n{\n"
		solid+="  \"id\" \""+(vmID as string)+"\"\n"
		vmID+=1	-- needs to be increased after every instance
		
		if face.count>3 do for x in face do solid+=createSide obj x size
		meshop.deleteFaces obj face	
		
		-- close solid
		solid+="}\n"
		-- restart loop
		face=#(1)
	)
	return solid
)

-----------------------------
-- Entities
-----------------------------

fn createEntity obj size=
(
	entity="entity\n{\n"
	entity+="  \"id\" \""+(vmID as string)+"\"\n"
	vmID+=1	-- needs to be increased after every instance
	
	entity+="  \"classname\" \""+obj.name+"\"\n"
	entity+="  \"angles\" \"0 0 0\"\n"
	entity+="  \"origin\" \""+(int(obj.position[1]*size)as string)+" "+(int(obj.position[2]*size)as string)+" "+(int(obj.position[3]*size)as string)+"\"\n"

	entity+="}\n"
	return entity
)

fn createLight obj size=
(
	case (classof obj) of
	(
		Omnilight:
		(
			entity="entity\n{\n"
			entity+="  \"id\" \""+(vmID as string)+"\"\n"
			vmID+=1	-- needs to be increased after every instance
			
			entity+="  \"classname\" \"light\"\n"
			entity+="  \"_light\" \""+(obj.rgb.r as string)+" "+(obj.rgb.g as string)+" "+(obj.rgb.b as string)+" "+((obj.multiplier*256) as string)+"\"\n"
			entity+="  \"_lightHDR\" \"-1 -1 -1 1\"\n"
			entity+="  \"_lightscaleHDR\" \"1\"\n"
			case obj.attenDecay of
			(
				1: entity+="  \"_constant_attn\" \"1\"\n"
				2: entity+="  \"_linear_attn\" \"1\"\n"
				3: entity+="  \"_quadratic_attn\" \"1\"\n"
			)
			entity+="  \"origin\" \""+(int(obj.position[1]*size)as string)+" "+(int(obj.position[2]*size)as string)+" "+(int(obj.position[3]*size)as string)+"\"\n"			
			entity+="}\n"
			return entity
		)
		freeSpot:
		(
			entity="entity\n{\n"
			entity+="  \"id\" \""+(vmID as string)+"\"\n"
			vmID+=1	-- needs to be increased after every instance
			
			entity+="  \"classname\" \"light_spot\"\n"
			entity+="  \"targetname\" \""+obj.name+"\"\n"
			
			-- angles is an own sorta topic
			if obj.rotation==(quat 0 0 0 1) then
			(
				rx="0"
				ry="-90"
				rz="90"
			)
			else
			(
				fsr=quatToEuler obj.rotation order:9
				--rx=((cutNumber fsr.x)) as string
				rx="0"
				ry=((cutNumber fsr.y)-90) as string
				rz=((cutNumber fsr.z)+270) as string
			)
						
			entity+="  \"angles\" \""+ry+" "+rz+" "+rx+"\"\n"
			entity+="  \"_light\" \""+(obj.rgb.r as string)+" "+(obj.rgb.g as string)+" "+(obj.rgb.b as string)+" "+((obj.multiplier*256) as string)+"\"\n"
			entity+="  \"_lightHDR\" \"-1 -1 -1 1\"\n"
			entity+="  \"_lightscaleHDR\" \"1\"\n"
			case obj.attenDecay of
			(
				1: entity+="  \"_constant_attn\" \"1\"\n"
				2: entity+="  \"_linear_attn\" \"1\"\n"
				3: entity+="  \"_quadratic_attn\" \"1\"\n"
			)
			-- hotspot and falloff angle defined differently aswell:
			-- max measures the whole angle, source only the angle from the axis of the spot.
			-- Therefore: 180 in 3D studio max mean the same as 90 in source 
			entity+="  \"_inner_cone\" \""+((obj.hotspot/2) as string)+"\"\n"
			entity+="  \"_cone\" \""+((obj.falloff/2) as string)+"\"\n"
			entity+="  \"origin\" \""+(int(obj.position[1]*size)as string)+" "+(int(obj.position[2]*size)as string)+" "+(int(obj.position[3]*size)as string)+"\"\n"
			entity+="}\n"
			return entity
		)
		default: return ""
	)
)

-----------------------------
-- World
-----------------------------

fn writeWorldToFile vmFile size exportHidden=
(	
	-- world header
	world="world\n{\n"
	world+="  \"id\" \""+(vmID as string)+"\"\n"
	vmID+=1	-- needs to be increased after every instance
  world+="  \"mapversion\" \"1\"\n"
  world+="  \"classname\" \"worldspawn\"\n"
  world+="  \"skyname\" \"sky_wasteland02\"\n"
  
  format world to:vmfile
	world="" 
  
	-- create solids
	for x in objects where (canConvertTo x Editable_Mesh) do
	(
		if (x.isHidden==false) or (exportHidden==true) do
		( 
			tempObj=convertToMesh (copy x)
			tempObj=cleanMesh tempObj
			world+=(createSolidMesh tempObj size)
			delete tempObj
		)
	)	
	world+="}\n" -- close world
	format world to:vmfile

	-- create entities	
	for x in objects do
	(
		if (classof x)==Dummy do format "%" (createEntity x size) to:vmfile
		if (superclassof x)==light do format "%" (createLight x size) to:vmfile
	)
	
	print "done"
	return true
)

-----------------------------
-- Rollouts
-----------------------------

rollout MAPExporterRollout "Valve Map File Exporter"
(
	edittext MapFile "File" width:250 text:"G:\\Projects\\Valve\\importexport\\try02.vmf" readOnly:true
	label scalerHelp "6,4 scales 10 to 64 valve units." across:2
	spinner scaler "Scale" range:[0,100,6.4] type:#float scale:0.1 fieldWidth:60 
	button bSetFilename "Set Mapfile Name" width:120 across:2
	button bExportVMF "Export Valve Map" width:120 
	button bRunMap "Run Map" width:120 across:2 --enabled:false
	button bExporter "Export Quake Map" width:120 enabled:false
	checkbox runGame "Run Game" checked:true across:2 --enabled:false
	checkbox exportHidden "Export hidden Geometry" checked:true
	
	on bSetFilename pressed do 
	(
		tempFileName=(getOpenFileName "Set Mapfile" filename:"G:\\Projects\\Valve\\importexport\\try02.vmf" types:"Valve Map File (*.vmf)|*.vmf|All Files|*.*" historyCategory:"VMFFiles")
		if tempFileName!=undefined do MapFile.text=tempFileName
	)

	on bExporter pressed do
	(
		temp=false
		temp=exportFile "G:\\Projects\\Valve\\importexport\\try01.3DS" #noPrompt selectedOnly:false
		doscommand "G:\\Projects\\Valve\\importexport\\3ds2map.exe try01.3DS\npause"
	)

	on bExportVMF pressed do
	(
		vmID=1 -- reset valve ID
		vmfilepath=MapFile.text
		vmfile=createFile vmfilepath
		format (createVersionInfo()) to:vmfile
		writeWorldToFile vmfile scaler.value exportHidden.checked
		close vmfile
	)

	on bRunMap pressed do
	(
		-- sourcepath="G:\\Projects\\Valve\\importexport\\try02"
		-- skdpath="G:\\Programme\\Steam\\steamapps\\badoli\\sourcesdk\\bin\\ep1\\bin"
		-- gamedir="G:\\Programme\\Steam\\steamapps\\badoli\\half-life 2\\hl2"
		-- bindir="G:\\Programme\\Steam\\steamapps\\badoli\\half-life 2"
		
		sourcepath=(getFilenamePath MapFile.text)+(getFilenameFile MapFile.text)
		INIFile=(GetDir #scripts)+"\\exportMapconf.ini"
		sdkpath=getINISetting INIFile "Paths" "sdkpath"
		gamedir=getINISetting INIFile "Paths" "gamepath"
		bindir=getINISetting INIFile "Paths" "binpath"
		destpath=getINISetting INIFile "Paths" "mappath"

		-- vbsp_exe -game $gamedir $path\$file
		-- G:\Programme\Steam\steamapps\badoli\sourcesdk\bin\ep1\bin\vbsp_exe
		doco=sdkpath+"bin\\ep1\\bin\\vbsp.exe -game \""+gamedir+"\" "+sourcepath+".vmf"
		doscommand doco
		-- vvis_exe -fast -game $gamedir $path\$file
		doco=sdkpath+"bin\\ep1\\bin\\vvis.exe -fast -game \""+gamedir+"\" "+sourcepath+".bsp"
		doscommand doco
		-- vlight_exe -bounce 2 -noextra -game $gamedir $path\$file
		doco=sdkpath+"bin\\ep1\\bin\\vrad.exe -bounce 2 -noextra -game \""+gamedir+"\" "+sourcepath+".bsp"
		doscommand doco 
		-- Copy File $path\$file.bsp $bspdir\$file.bsp
		try delete destpath catch()
		doco="copy "+(sourcepath+".bsp")+" \""+destpath+"\""
		doscommand doco
		if runGame.checked==true do
		(
			-- start game
			-- $game_exe -allowdebug -game $gamedir +map $file
			doco="\""+bindir+"\" -allowdebug -window -nologos -map try02.bsp"
			print "Starting game..."
			doscommand doco
		)
		print "...and here we come again!"
	)
	-- 
	-- "G:\\Programme\\Steam\\steamapps\\badoli\\half-life 2\\hl2\\maps\\"
	-- "-dev -window -nointro -console -allowdebug -game $gamedir +map $file"
)

rollout gameSDKpathsRollout "Game Paths"
(
	edittext steampath "Insert Path of SourceSDK here:" text:"C:\\Programme\\Steam\\NoUserYet\\sourceskd\\" labelOnTop:true
	dropdownlist gameSelector "Game:" items:#("Halflife 2","HL2 Episode 2","team fortress 2") selection:1 enabled:false
	edittext gamepath "Gamepath" readOnly:true labelOnTop:true -- the path where the GameInfo.txt lies
	edittext binpath "Binary" readOnly:true labelOnTop:true -- the path where the .EXE lies
	edittext mappath "Mappath" readOnly:true labelOnTop:true -- the path where the maps are copied to
	edittext sdkpath "SDK Path" readOnly:true labelOnTop:true -- the path where SDK lies
	
	on gameSDKpathsRollout open do
	(
		INIFile=(GetDir #scripts)+"\\exportMapconf.ini"
		if (existFile INIFile) do
		(
			gamepath.text=getINISetting INIFile "Paths" "gamepath"
			binpath.text=getINISetting INIFile "Paths" "binpath"
			mappath.text=getINISetting INIFile "Paths" "mappath"
			sdkpath.text=getINISetting INIFile "Paths" "sdkpath"
			steampath.text=sdkpath.text
		)
	)
	
	on steampath changed VMFpathname do
	(
		if VMFpathname[VMFpathname.count]!="\\" do VMFpathname+="\\"
		if (existDir VMFpathname)==true and (matchPattern VMFpathname pattern:"*sourcesdk\\") then
		(
			case gameSelector.selected of
			(
				"Halflife 2":
				(
					gamepath.text=(substituteString VMFpathname "sourcesdk\\" "")+"half-life 2\\hl2"
					binpath.text=(substituteString VMFpathname "sourcesdk\\" "")+"half-life 2\\hl2.exe"
					mappath.text=(substituteString VMFpathname "sourcesdk\\" "")+"half-life 2\\hl2\\maps\\"
					sdkpath.text=VMFpathname
				)
			)
			INIFile=(GetDir #scripts)+"\\exportMapconf.ini"
			setINISetting INIFile "Paths" "gamepath" gamepath.text
			setINISetting INIFile "Paths" "binpath" binpath.text
			setINISetting INIFile "Paths" "mappath" mappath.text
			setINISetting INIFile "Paths" "sdkpath" sdkpath.text
		)
		else
		(
			gamepath.text=""
			binpath.text=""
			mappath.text=""
			sdkpath.text=""
		)			
	)
)

rollout notesRollout "Notes"
(
	edittext notes "Notes" labelOnTop:true height:60
)

rollout debug "debug"
(
	edittext debugmsg "Debug"
	spinner moveFirstVerticesUV "Move UV" range:[0,1,.5] scale:0.01
	button apply "Apply Value"
	
	on moveFirstVerticesUV changed cVal do
	(
		debugmsg.text=(cVal as string)
		if selection.count>0 and (classof selection[1])==Editable_Poly do
		(
			mfaces=polyop.getMapFace selection[1] 1 1
			polyop.setMapVert selection[1] 1 mfaces[1] [1,cVal,0]
			update selection[1] geometry:true
		)

		if selection.count>0 and (classof selection[1])==Editable_Mesh do
		(
			mfaces=meshop.getMapFace selection[1] 1 1
			meshop.setMapVert selection[1] 1 mfaces[1] [1,cVal,0]
			update selection[1] geometry:true
		)
	)
)

try (closeRolloutFloater MAPExporterWin) catch()
MAPExporterWin = newRolloutFloater ("Map Exporter") 300 180 970 310
addRollout MAPExporterRollout MAPExporterWin rolledup:false
addRollout gameSDKpathsRollout MAPExporterWin rolledup:true
-- addRollout notesRollout MAPExporterWin rolledup:true
-- addRollout debug MAPExporterWin rolledup:false